/********************************************************************************
*                                                                               *
*         I n t e r - T h r e a d    M e s s a g i n g    C h a n n e l         *
*                                                                               *
*********************************************************************************
* Copyright (C) 2006 by Jeroen van der Zijp.   All Rights Reserved.             *
*********************************************************************************
* This library is free software; you can redistribute it and/or                 *
* modify it under the terms of the GNU Lesser General Public                    *
* License as published by the Free Software Foundation; either                  *
* version 2.1 of the License, or (at your option) any later version.            *
*                                                                               *
* This library is distributed in the hope that it will be useful,               *
* but WITHOUT ANY WARRANTY; without even the implied warranty of                *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
* Lesser General Public License for more details.                               *
*                                                                               *
* You should have received a copy of the GNU Lesser General Public              *
* License along with this library; if not, write to the Free Software           *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
*********************************************************************************
* $Id: FXDirWatch.cpp,v 1.11 2006/11/15 07:12:07 fox Exp $                      *
********************************************************************************/
#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "FXException.h"
#include "FXHash.h"
#include "FXThread.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXSize.h"
#include "FXPoint.h"
#include "FXRectangle.h"
#include "FXRegistry.h"
#include "FXApp.h"
#include "FXDirWatch.h"

#ifdef __linux
#include <sys/syscall.h>
#ifdef SYS_inotify_init
#include <sys/inotify.h>
#define HAVE_INOTIFY 1
#endif
#endif



/*
  Notes:
  - Make this more elegant.
*/


// Maximum message size
#define MAXMESSAGE 8192

// Bad handle value
#ifdef WIN32
#define BadHandle  INVALID_HANDLE_VALUE
#else
#define BadHandle  -1
#endif


using namespace FX;


/*******************************************************************************/

namespace FX {


// Map
FXDEFMAP(FXDirWatch) FXDirWatchMap[]={
  FXMAPFUNC(SEL_IO_READ,FXDirWatch::ID_IO_READ,FXDirWatch::onMessage)
  };


// Object implementation
FXIMPLEMENT(FXDirWatch,FXObject,FXDirWatchMap,ARRAYNUMBER(FXDirWatchMap));


// Initialize to empty
FXDirWatch::FXDirWatch():app((FXApp*)-1L),hnd(BadHandle){
  }


// Add handler to application
FXDirWatch::FXDirWatch(FXApp* a):app(a){
#ifdef WIN32
//  SECURITY_ATTRIBUTES sa;
//  sa.nLength=sizeof(sa);
//  sa.lpSecurityDescriptor=NULL;         // Default ACL
//  sa.bInheritHandle=false;              // Don't inherit handles
//  if(::CreatePipe(&fd[0],&fd[1],&sa,0)==0){ throw FXResourceException("unable to create pipe."); }
//  app->addInput(this,ID_IO_READ,fd[0],INPUT_READ);
#else
//  hnd=::inotify_init();
  if(hnd<0){ throw FXResourceException("unable to create directory watcher."); }
  ::fcntl(hnd,F_SETFD,FD_CLOEXEC);
  app->addInput(this,ID_IO_READ,hnd,INPUT_READ);
#endif
  }



// Watch directory
FXbool FXDirWatch::watch(const FXString& dir){
#ifdef WIN32
  return false;
#else
//  return ::inotify_add_watch(hnd,dir.text(),IN_MODIFY|IN_CREATE|IN_DELETE|IN_ONESHOT)>=0;
#endif
  }


// Fire signal message to target
long FXDirWatch::onMessage(FXObject*,FXSelector,void*){
#ifdef WIN32
#else
/*
  FXchar buf[256*sizeof(inotify_event)];
  inotify_event *event;
  FXint len, i = 0;
  FXTRACE((1,"onMessage\n"));
  len=::read(hnd,buf,sizeof(buf));
  if(0<=len){
    for(i=0; i<len; ) {
      event=(inotify_event*)&buf[i];
printf ("wd=%d mask=%u cookie=%u len=%u\n",event->wd, event->mask,event->cookie, event->len);
if(event->len) printf ("name=%s\n", event->name);
      i+=sizeof(inotify_event)+event->len;
      }
    }
*/
#endif
  return 1;
  }


// Remove handler from application
FXDirWatch::~FXDirWatch(){
#ifdef WIN32
  app->removeInput(hnd,INPUT_READ);
  ::CloseHandle(hnd);
#else
  app->removeInput(hnd,INPUT_READ);
  ::close(hnd);
#endif
  app=(FXApp*)-1L;
  }

//HANDLE FindFirstChangeNotification(
// LPCTSTR lpPathName,  // directory name
// BOOL bWatchSubtree,  // monitoring option
// DWORD dwNotifyFilter  // filter conditions
//);
//
//The HANDLE can be passed to FXApp::addInput(), and you'll be notified
//for all events as specified in dwNotifyFilter.

#if 0
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <tchar.h>

void RefreshDirectory(LPTSTR);
void RefreshTree(LPTSTR);

void WatchDirectory(LPTSTR lpDir)
{
   DWORD dwWaitStatus;
   HANDLE dwChangeHandles[2];
   TCHAR lpDrive[4];
   TCHAR lpFile[_MAX_FNAME];
   TCHAR lpExt[_MAX_EXT];

   _tsplitpath(lpDir, lpDrive, NULL, lpFile, lpExt);

   lpDrive[2] = (TCHAR)'\\';
   lpDrive[3] = (TCHAR)'\0';

// Watch the directory for file creation and deletion.

   dwChangeHandles[0] = FindFirstChangeNotification(
      lpDir,                         // directory to watch
      FALSE,                         // do not watch subtree
      FILE_NOTIFY_CHANGE_FILE_NAME); // watch file name changes

   if (dwChangeHandles[0] == INVALID_HANDLE_VALUE)
      ExitProcess(GetLastError());

// Watch the subtree for directory creation and deletion.

   dwChangeHandles[1] = FindFirstChangeNotification(
      lpDrive,                       // directory to watch
      TRUE,                          // watch the subtree
      FILE_NOTIFY_CHANGE_DIR_NAME);  // watch dir. name changes

   if (dwChangeHandles[1] == INVALID_HANDLE_VALUE)
      ExitProcess(GetLastError());

// Change notification is set. Now wait on both notification
// handles and refresh accordingly.

   while (TRUE)
   {
   // Wait for notification.

      dwWaitStatus = WaitForMultipleObjects(2, dwChangeHandles,
         FALSE, INFINITE);

      switch (dwWaitStatus)
      {
         case WAIT_OBJECT_0:

         // A file was created or deleted in the directory.
         // Refresh this directory and restart the notification.

            RefreshDirectory(lpDir);
            if ( FindNextChangeNotification(
                    dwChangeHandles[0]) == FALSE )
                ExitProcess(GetLastError());
            break;

         case WAIT_OBJECT_0 + 1:

         // A directory was created or deleted in the subtree.
         // Refresh the tree and restart the notification.

            RefreshTree(lpDrive);
            if (FindNextChangeNotification(
                    dwChangeHandles[1]) == FALSE)
                ExitProcess(GetLastError());
            break;

        default:
            ExitProcess(GetLastError());
      }
   }
}

void RefreshDirectory(LPTSTR lpDir)
{
   _tprintf(TEXT("Refresh the directory (%s).\n"), lpDir);
}

void RefreshTree(LPTSTR lpDrive)
{
   _tprintf(TEXT("Refresh the directory tree (%s).\n"), lpDrive);
}
#endif

}
